import { JSONSchema7 } from "json-schema";
import { OrderSensitiveQueryParams } from "../../../common/model/OrderSensitiveQueryParams";
import { QueryParamDefinitionMap } from "../../../plugins/qsToJsonMapper/model/QueryParamDefinitionMap";
import { QueryStringToJsonResult } from "../../../plugins/qsToJsonMapper/model/QueryStringToJsonResult";
export declare class QueryStringToJsonFunction {
    private readonly schema;
    private readonly paramDefinitions;
    private readonly objectAutoCompleter;
    private readonly objectRasterizer;
    constructor(schema: JSONSchema7, paramDefinitions: QueryParamDefinitionMap);
    /**
     * IMPORTANT: returned object may be invalid, due to being incomplete (as there may not have been enough data given in
     * the query params) or may be invalid due to extended validation rules like regex patterns and min/max/minItems/maxItems
     * rules not being checked. Essentially, we only check basic typing and object structure, up to the point it effects
     * type union resolution, but nothing beyond that.
     */
    toJson(queryString: OrderSensitiveQueryParams): QueryStringToJsonResult;
    private setQueryParam;
    /**
     * Returns the updated 'object' or 'undefined' if it's impossible to set the given field into this object.
     */
    private trySet;
    /**
     * If the array is empty, or we're adding a scalar element, or we forcefully want a new array element, then create
     * a new element, else return the last element, so it can be updated in-place (which we always want to attempt to do
     * first if it's a complex object, i.e. attempt augment the last element, and on failure, use forceAppend to add new).
     */
    private calculateNextArrayElementIndex;
    private tryParseParamValue;
}
