import { JSONSchema7 } from "json-schema";
import { JsonValue, NonEmpty, NonEmptyArray } from "@upload-io/upload-shared-iso";
import { NormalizedJsonSchema } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/model/ObjectWithMetadata";

export class JsonSchemaUtils {
  /**
   * Assumes no $ref in schema.
   */
  static normalizePolymorphisms(schema: JSONSchema7): NonEmptyArray<NormalizedJsonSchema> {
    if (schema.anyOf !== undefined) {
      return NonEmpty(schema.anyOf.flatMap(x => this.normalizePolymorphisms(x as JSONSchema7)));
    }

    if (schema.oneOf !== undefined) {
      return NonEmpty(schema.oneOf.flatMap(x => this.normalizePolymorphisms(x as JSONSchema7)));
    }

    if (schema.allOf !== undefined) {
      throw new Error(
        "allOf currently not supported or expected: please create a single merged type instead of using 'allOf'."
      );
    }

    return [schema];
  }

  static passesBasicTypeCheck(schema: JSONSchema7 & { nullable?: boolean }, value: JsonValue): boolean {
    if (value === null) {
      return schema.nullable === true;
    }

    const ownType = schema.type;
    const actualType = typeof value;
    const isTypeMatch =
      (ownType === "array" && Array.isArray(value)) ||
      ownType === actualType ||
      (ownType === "integer" && actualType === "number");
    const isInEnum = schema.enum === undefined || schema.enum.includes(value);
    return isTypeMatch && isInEnum;
  }
}
